
require 'fileutils'

require 'ygg/provisioner/task'
require 'ygg/provisioner/tools'
require 'ygg/vhosts/provisioner/vhost'

module Ygg
module Vhosts
module Provisioner

class Task < Ygg::Provisioner::Task
  class VhostNotFound < PermanentFailure ; end
  class ProvisionFail < TemporaryFailure ; end

  def find_vhost(id)
    Vhost.find(id)
  rescue Vhost::NotFound
    raise VhostNotFound
  end

  operation :create do
    help_caption 'create'
    help_text <<-EOT.unindent
      parameters:
        request:    The whole request
    EOT
  end

  def create
    res = nil
    data = request_data
    data[:state] = 'active'
    vhost = Vhost.new(data)
    vhost.update_state({state: 'active'})

    log.info "######### CREATE #{vhost.username} ############"

    vhost.logging_on(plog) do
      vhost.save!
      res = vhost.provision!
      raise ProvisionFail.new('Provisioning failed') unless res
    end

    log.info "######### CREATE #{vhost.username} DONE #######"

    res
  end

  operation :update do
    help_caption 'update'
    help_text <<-EOT.unindent
      parameters:
        request:    The whole request
    EOT
  end

  def update
    data = request_data
    begin
      vhost = find_vhost(data[:id])
      vhost.update_config(data)
    rescue
      # In case of corrupted local database
      vhost = Vhost.new(data)
      vhost.update_state({state: 'active'})
    end

    log.info "######### UPDATING #{vhost.username} ############"

    res = nil
    vhost.logging_on(plog) do
      vhost.save!
      res = if vhost.state == 'active'
        vhost.provision!
      else
        vhost.deprovision!
      end
      puts res.inspect
      raise ProvisionFail.new('Provisioning failed') unless res
    end

    log.info "######### UPDATING #{vhost.username} DONE #######"

    res
  end

  operation :delete do
    help_caption 'deprovision the vhost'
    help_text <<-EOT.unindent
      parameters:
        id
    EOT
  end

  def delete(id)
    begin
      vhost = find_vhost(id)
    rescue VhostNotFound
      log.warn "######### DELETE #{id} SILENTLY IGNORED SINCE NOT FOUND ############"
      return nil
    end
    vhost.update_state({state: 'deleted'})

    log.info "######### DELETE #{vhost.username} ############"

    vhost.logging_on(plog) do
      vhost.save!
      res = vhost.deprovision!
      raise ProvisionFail.new('Provisioning failed') unless res
    end

    log.info "######### DELETE #{vhost.username} DONE #######"

    nil
  end

  operation :suspend do
    help_caption 'suspend'
    help_text <<-EOT.unindent
      parameters:
        id
    EOT
  end

  def suspend(id)
    begin
      vhost = find_vhost(id)
    rescue VhostNotFound
      log.warn "######### SUSPEND #{id} SILENTLY IGNORED SINCE NOT FOUND ############"
      return nil
    end
    vhost.update_state({state: 'suspended'})

    log.info "######### SUSPEND #{vhost.username} ############"

    res = nil
    vhost.logging_on(plog) do
      vhost.save!
      res = vhost.deprovision!
      raise ProvisionFail.new('Provisioning failed') unless res
    end

    log.info "######### SUSPEND #{vhost.username} DONE #######"

    res
  end

  operation :resume do
    help_caption 'resume'
    help_text <<-EOT.unindent
      parameters:
        id
    EOT
  end

  def resume(id)
    begin
      vhost = find_vhost(id)
    rescue VhostNotFound
      log.warn "######### RESUME #{id} SILENTLY IGNORED SINCE NOT FOUND ############"
      return nil
    end
    vhost.update_state({state: 'active'})
    
    log.info "######### RESUME #{vhost.username} ############"

    res = nil
    vhost.logging_on(plog) do
      vhost.save!
      res = vhost.provision!
      raise ProvisionFail.new('Provisioning failed') unless res
    end
    
    log.info "######### RESUME #{vhost.username} DONE #######"
    
    res
  end

  operation :letsencrypt do
    help_caption 'letsencrypt'
    help_text <<-EOT.unindent
      parameters:
        id
    EOT
  end

  def letsencrypt(id)
    begin
      vhost = find_vhost(id)
    rescue VhostNotFound
      log.warn "######### LETSENCRYPT #{id} SILENTLY IGNORED SINCE NOT FOUND ############"
      return nil
    end
    vhost.update_state({state: 'active'})
    
    log.info "######### UPDATE LETSENCRYPT #{vhost.username} ############"

    res = nil
    vhost.logging_on(plog) do
      vhost.save!
      res = vhost.provision_letsencrypt!
      raise ProvisionFail.new('Provisioning failed') unless res
    end
    
    log.info "######### UPDATE LETSENCRYPT #{vhost.username} DONE #######"
    
    res
  end
end

end
end
end
